//+------------------------------------------------------------------+
//|                                                       xpMaEA.mq4 |
//|                                                     Coders' Guru |
//|                                            http://www.xpworx.com |
//+------------------------------------------------------------------+

#property copyright "Coders' Guru"
#property link      "http://www.xpworx.com"
//Last Modification = 2010.10.13

#define MODE_DEMA    4
#define MODE_TEMA    5
#define MODE_T3MA    6
#define MODE_JMA     7
#define MODE_HMA     8
#define MODE_DECEMA  9
#define MODE_SALT    10

//---- Trades limits
extern   double      Lots                       = 1;
extern   int         TakeProfit                 = 100;
extern   int         TrailingStop               = 50; 
extern   int         StopLoss                   = 50;
extern   int         Slippage                   = 5;
extern   bool        UseMoneyManagement         = true;
extern   double      MaximumRisk                = 0.1;
extern   int         MagicNumber                = 12345;
extern   bool        AutoGenerateMagicNumber    = false;


//--- xpMA Indicators settings
extern   int         TimeFrame            = 0;
extern   int         MA_Period            = 24;
extern   int         MA_Type              = MODE_TEMA;
extern   int         MA_Applied           = PRICE_CLOSE;
extern   double      T3MA_VolumeFactor    = 0.8;
extern   double      JMA_Phase            = 0;
extern   int         Step_Period          = 4;


//--- Global variables
string   ExpertComment        = "xpMaEA v2";
int      NumberOfTries        = 5;
double   mPoint               = 0.0001;

//+------------------------------------------------------------------
int init()
{
   if(AutoGenerateMagicNumber) MagicNumber = GetMagicNumber(558854);
   
   mPoint = GetPoint();
   
   return(0);
}

int deinit() 
{
   return(0);
}
//+------------------------------------------------------------------

bool isNewSymbol(string current_symbol)
  {
   //loop through all the opened order and compare the symbols
   int total  = OrdersTotal();
   for(int cnt = 0 ; cnt < total ; cnt++)
   {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      string selected_symbol = OrderSymbol();
      if (current_symbol == selected_symbol && OrderMagicNumber()==MagicNumber)
      return (False);
    }
    return (True);
}

//+------------------------------------------------------------------+
int start()
{
   
   double  signal = iCustom(NULL,0,"xpMA",TimeFrame,MA_Period,MA_Type,MA_Applied,T3MA_VolumeFactor,JMA_Phase,Step_Period,false,3,0);
   
   bool BuyCondition = false , SellCondition = false , CloseBuyCondition = false , CloseSellCondition = false; 
   
   if (signal==1)
       BuyCondition = true;
       
   if (signal==-1)
      SellCondition = true;
   
   if (BuyCondition)
      CloseSellCondition = true;
   
   if (SellCondition)
      CloseBuyCondition = true;
      
   
   if(UseMoneyManagement)
   Lots=NormalizeDouble(AccountFreeMargin()*MaximumRisk/1000.0,1);
   
   if(OrdersTotal() < 1 || isNewSymbol(Symbol())) 
   {
      int ticket;
      
      if(BuyCondition) //<-- BUY condition
      {
         ticket = OpenOrder(Symbol(),OP_BUY,Lots,Slippage,StopLoss,TakeProfit,MagicNumber,WindowExpertName(),5);
      }
      if(SellCondition) //<-- SELL condition
      {
         ticket = OpenOrder(Symbol(),OP_SELL,Lots,Slippage,StopLoss,TakeProfit,MagicNumber,WindowExpertName(),5);
      }
   }
     
   for(int cnt = 0 ; cnt < OrdersTotal() ; cnt++)
   {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if (OrderSymbol() == Symbol() && OrderMagicNumber()== MagicNumber)
      {
         if(CloseBuyCondition && OrderType()==OP_BUY) CloseOrder(OrderTicket()); 
         if(CloseSellCondition && OrderType()==OP_SELL) CloseOrder(OrderTicket()); 
      }
   }
   
   if(TrailingStop>0)TrailOrders(TrailingStop,MagicNumber);
   
   return(0);
}
//+------------------------------------------------------------------+

int OpenOrder(string TradeSymbol, int TradeType, double TradeLot, int TradeSlippage, int TradeStopLoss, int TradeTakeProfit, int TradeMagic, string TradeComment = "", int TriesCount = 5)
{
   int ticket=0;
   int err=0;
   int num = 0;
   
   RefreshRates();
   double pt = mPoint;
   double ask = MarketInfo(TradeSymbol,MODE_ASK);
   double bid = MarketInfo(TradeSymbol,MODE_BID);
   
   if(TradeType == OP_BUY)
   {
      for(num = 0 ; num < TriesCount ; num++)
      {
         if(TradeStopLoss>0 && TradeTakeProfit>0)
            ticket=OrderSend(TradeSymbol,OP_BUY,TradeLot,ask,TradeSlippage,ask-TradeStopLoss*pt,ask+TradeTakeProfit*pt,TradeComment,TradeMagic,0,Green);
         if(TradeStopLoss==0 && TradeTakeProfit==0)
            ticket=OrderSend(TradeSymbol,OP_BUY,TradeLot,ask,TradeSlippage,0,0,TradeComment,TradeMagic,0,Green);
         if(TradeStopLoss>0 && TradeTakeProfit==0)
            ticket=OrderSend(TradeSymbol,OP_BUY,TradeLot,ask,TradeSlippage,ask-TradeStopLoss*pt,0,TradeComment,TradeMagic,0,Green);
         if(TradeStopLoss==0 && TradeTakeProfit>0)
            ticket=OrderSend(TradeSymbol,OP_BUY,TradeLot,ask,TradeSlippage,0,ask+TradeTakeProfit*pt,TradeComment,TradeMagic,0,Green);
         
         err=GetLastError();
         if(err==4 || err==6 || err==8 || err==128 || err==129 || err==135 || err==136 || err==137 || err==138 || err==141 || err==146)
         { 
            Sleep(5000);
            continue;
         }
         else
         {
            break;
         }
      }   
   }
   
   if(TradeType == OP_SELL)
   {   
      for(num = 0 ; num < TriesCount ; num++)
      {
         if(TradeStopLoss>0 && TradeTakeProfit>0)
            ticket=OrderSend(TradeSymbol,OP_SELL,TradeLot,bid,TradeSlippage,bid+TradeStopLoss*pt,bid-TradeTakeProfit*pt,TradeComment,TradeMagic,0,Red);
         if(TradeStopLoss==0 && TradeTakeProfit==0)
            ticket=OrderSend(TradeSymbol,OP_SELL,TradeLot,bid,TradeSlippage,0,0,TradeComment,TradeMagic,0,Red);
         if(TradeStopLoss>0 && TradeTakeProfit==0)
            ticket=OrderSend(TradeSymbol,OP_SELL,TradeLot,bid,TradeSlippage,bid+TradeStopLoss*pt,0,TradeComment,TradeMagic,0,Red);
         if(TradeStopLoss==0 && TradeTakeProfit>0)
            ticket=OrderSend(TradeSymbol,OP_SELL,TradeLot,bid,TradeSlippage,0,bid-TradeTakeProfit*pt,TradeComment,TradeMagic,0,Red);

         err=GetLastError();
         if(err==4 || err==6 || err==8 || err==128 || err==129 || err==135 || err==136 || err==137 || err==138 || err==141 || err==146)
         { 
            Sleep(5000);
            continue;
         }
         else
         {
            break;
         }
      }   
   }  
   return(ticket);
}

void CloseOrder(int ticket, double lot = 0)
{
   bool result;
   int tries = 5;
   int pause = 500;
   double ask = NormalizeDouble(Ask,Digits);
   double bid = NormalizeDouble(Bid,Digits);
   if(OrderSelect(ticket,SELECT_BY_TICKET,MODE_TRADES))
   {
      if(OrderType()==OP_BUY)
      {
         for(int c = 0 ; c <= 5 ; c++)
         {
            if(lot==0) result = OrderClose(OrderTicket(),OrderLots(),bid,5,Violet);
            else result = OrderClose(OrderTicket(),lot,bid,5,Violet);
            if(result==true) break; 
            else
            {
               Sleep(pause);
               continue;
            }
         }
      }
      if(OrderType()==OP_SELL)
      {
         for(c = 0 ; c <= 5 ; c++)
         {
            if(lot==0) result = OrderClose(OrderTicket(),OrderLots(),ask,5,Violet);
            else result = OrderClose(OrderTicket(),lot,ask,5,Violet);
            if(result==true) break; 
            else
            {
               Sleep(pause);
               continue;
            }
         }
      }
   }
   return(result);
}

int TrailOrders(int ts, int magic)
{
   if(ts<=0) return(0);
 
   int result = -1;
   double point = mPoint;//GetPoint();
 
   for(int cnt=0;cnt<OrdersTotal();cnt++) 
   {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if(OrderType()==OP_SELL && OrderSymbol()==Symbol()  && OrderMagicNumber()==magic)  
      { 
         if (OrderStopLoss()==0)
         {
            result = OrderModify(OrderTicket(),OrderOpenPrice(),Ask+point*ts,OrderTakeProfit(),0,Red);
         }    
 
         if(OrderOpenPrice()-Ask>point*ts)
         {
            if(OrderStopLoss()>(Ask+point*ts)+point)
            {
               result = OrderModify(OrderTicket(),OrderOpenPrice(),Ask+point*ts,OrderTakeProfit(),0,Red);
            }
         }
      }
        
      if(OrderType()==OP_BUY && OrderSymbol()==Symbol()  && OrderMagicNumber()==magic)  
      { 
         if (OrderStopLoss()==0)
         {
            result = OrderModify(OrderTicket(),OrderOpenPrice(),Bid-point*ts,OrderTakeProfit(),0,Green);
         }
             
         if(Bid-OrderOpenPrice()>point*ts)
         {
            if(OrderStopLoss()<(Bid-point*ts)-point)
            {
               result = OrderModify(OrderTicket(),OrderOpenPrice(),Bid-point*ts,OrderTakeProfit(),0,Green);
            }
         }
      }
   }
   
   return(result);
}

int GetMagicNumber(int base)
{
   int Reference = 0;
   string symbol = StringUpperCase(Symbol());
   if (StringFind(symbol,"AUDCAD")>-1) Reference = base + 1001 + Period();
   if (StringFind(symbol,"AUDJPY")>-1) Reference = base + 2002 + Period();
   if (StringFind(symbol,"AUDNZD")>-1) Reference = base + 3003 + Period();
   if (StringFind(symbol,"AUDUSD")>-1) Reference = base + 4004 + Period();
   if (StringFind(symbol,"CHFJPY")>-1) Reference = base + 5005 + Period();
   if (StringFind(symbol,"EURAUD")>-1) Reference = base + 6006 + Period();
   if (StringFind(symbol,"EURCAD")>-1) Reference = base + 7007 + Period();
   if (StringFind(symbol,"EURCHF")>-1) Reference = base + 8008 + Period();
   if (StringFind(symbol,"EURGBP")>-1) Reference = base + 9009 + Period();
   if (StringFind(symbol,"EURJPY")>-1) Reference = base + 1010 + Period();
   if (StringFind(symbol,"EURUSD")>-1) Reference = base + 1111 + Period();
   if (StringFind(symbol,"GBPCHF")>-1) Reference = base + 1212 + Period();
   if (StringFind(symbol,"GBPJPY")>-1) Reference = base + 1313 + Period();
   if (StringFind(symbol,"GBPUSD")>-1) Reference = base + 1414 + Period();
   if (StringFind(symbol,"NZDJPY")>-1) Reference = base + 1515 + Period();
   if (StringFind(symbol,"NZDUSD")>-1) Reference = base + 1616 + Period();
   if (StringFind(symbol,"USDCHF")>-1) Reference = base + 1717 + Period();
   if (StringFind(symbol,"USDJPY")>-1) Reference = base + 1818 + Period();
   if (StringFind(symbol,"USDCAD")>-1) Reference = base + 1919 + Period();
   if (Reference == 0) Reference = base + 2020 + Period();
   return(Reference);
}

string StringUpperCase(string str)
{
   int s = StringLen(str);
   int chr = 0;
   string temp;
   for (int c = 0 ; c < s ; c++)
   {
      chr = StringGetChar(str,c);
      if (chr >= 97 && chr <=122) chr = chr - 32;
      temp = temp + CharToStr(chr);
   }
   return (temp);  
}

double GetPoint(string symbol = "")
{
   if(symbol=="" || symbol == Symbol())
   {
      if(Point==0.00001) return(0.0001);
      else if(Point==0.001) return(0.01);
      else return(Point);
   }
   else
   {
      RefreshRates();
      double tPoint = MarketInfo(symbol,MODE_POINT);
      if(tPoint==0.00001) return(0.0001);
      else if(tPoint==0.001) return(0.01);
      else return(tPoint);
   }
}



